package fi.cc.android;

import java.io.IOException;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.ArrayList;

public class ConnectionEngine extends Thread{
	private ServerSocket serverSocket=null; //The listening socket
	private Socket socket = null;			//Socket for communicating with other user
    private boolean connected = false;
    public boolean isServer=false;
    private final String defaultIpAddress = "127.0.0.1";  	// Default server address
    private final int defaultPort = 4444;					// Default server port
	private ConnectionCallback callBack=null;

	public void setCallBack(ConnectionCallback c){
		callBack=c;
	}
	
    public boolean startServer(int port){ 	

        try {
                 if (serverSocket == null) {
                       serverSocket = new ServerSocket(port);
                  }
                  if (serverSocket!=null){
                	  if(callBack!=null){
                     	 callBack.infoMessage("Listening on port: " + port);
                      }
                	  isServer=true;
                	  this.start();
                  }
             
        } catch (IOException e) {
             System.err.println("Could not listen on port: " + port);
             if(callBack!=null){
            	 callBack.infoMessage("Could not listen on port: " + port);
             }
             return false;
        }
    	return true;
    }
    
    public boolean startServer(){ 	

        return this.startServer(defaultPort);
    }
    
    public boolean createConnection(String addr, int port){
    	
    	try {
            socket = new Socket(addr, port);
    	} catch (UnknownHostException e) {
    		System.err.println("Unknown host" + addr);
    		if(callBack!=null){
           	 	callBack.infoMessage("Unknown host" + addr);
            }
    	} catch (IOException e) {
     	  	System.err.println("Couldn't get I/O for the connection to: " + addr);
     	  	if(callBack!=null){
           	 	callBack.infoMessage("Couldn't get I/O for the connection to: " + addr);
            }
    	}
    	if(socket!=null){
    		connected = true;
    		this.start();
    		if(callBack!=null){
    			callBack.connectionCreated();
           	 	callBack.infoMessage("Connection created ");
            }
    		return true;
    	}
    	else
    		return false;
    }
    
    public void sendMessage(byte[] data){
    	if (connected){
    		int length=data.length;
    		byte[] msg=new byte[data.length+4];
    		
    		msg[0] = (byte) ((length >> 24) & 0xff);
        	msg[1] = (byte) ((length >> 16) & 0xff);
        	msg[2] = (byte) ((length >> 8) & 0xff);
        	msg[3] = (byte) ((length) & 0xff);
    		
    		
    		System.arraycopy(data, 0, msg, 4, data.length);
	    	try {
				socket.getOutputStream().write(msg);
			} catch (IOException e) {
				System.err.println("Failed to send message");
	     	  	if(callBack!=null){
	           	 	callBack.infoMessage("Failed to send message");
	            }
			}
    	}
    }
    public void sendMessage(String msg){
    	PrintWriter out;
    
    	if (connected){
			try {
				out = new PrintWriter(socket.getOutputStream(), true);
				out.println(msg);
			} catch (IOException e) {
				System.err.println("Failed to send message");
	     	  	if(callBack!=null){
	           	 	callBack.infoMessage("Failed to send message");
	            }
			}
		}
    	else
    		callBack.infoMessage("Not connected");
        
    }
    
    public void CloseConnection(){
    	try {
			socket.close();
		} catch (IOException e) {
			System.err.println("Failed to close connection");
     	  	if(callBack!=null){
           	 	callBack.infoMessage("Failed to close connection");
            }
		}
    }
    
    private int getInt (byte[] buf){
    	if (buf == null) {
    		return -1;
    	}
      	if (buf.length!= 4) {
      		 return -1;
      	}
      	int r=0;
      	
      	for (int i =0; i<buf.length; i++){
      		r <<= 8; 
      		r |= (buf [i] & 0x000000ff);
      	}
      	return r;
    }

    public void run() {
    	while(true){
    		
    	
	    	if(!connected){
	    		try {
					socket = serverSocket.accept();
				} catch (IOException e) {
					System.err.println("Failed to accept connection");
					if(callBack!=null){
		           	 	callBack.infoMessage("Failed to accept connection");
		            }
				}
				if(callBack!=null){
					callBack.connectionCreated();
	           	 	callBack.infoMessage("Connected");
	            }
	    		connected = true;
	    	}
	    	else{
		     try {
		    	 byte[] msglength=new byte[4];
		    	 socket.getInputStream().read(msglength);
		    	 int length=getInt(msglength);
		    	 byte[] input=new byte[length];
		    	 socket.getInputStream().read(input);
 		    	 callBack.dataMessage(input);
		         
		    	 /*BufferedReader in = new BufferedReader(
		                        new InputStreamReader(
		                        socket.getInputStream()));
	
		         String inputLine = in.readLine();
		          callBack.dataMessage(inputLine);*/
	
		     } catch (IOException e) {
		    	 System.err.println("Failed to receive");
		     	 if(callBack!=null){
		           	 	callBack.infoMessage("Failed to receive");
		         }
		     }
	    	}
    	}
    }
}
